# Package Constants

# Meraki dashboard API key, set either at instantiation or as an environment variable
API_KEY_ENVIRONMENT_VARIABLE = 'MERAKI_DASHBOARD_API_KEY'

# Base URL preceding all endpoint resources
DEFAULT_BASE_URL = 'https://api.meraki.com/api/v0'

# Maximum number of seconds for each API call
SINGLE_REQUEST_TIMEOUT = 60

# Path for TLS/SSL certificate verification if behind local proxy
CERTIFICATE_PATH = ''

# Retry if 429 rate limit error encountered?
WAIT_ON_RATE_LIMIT = True

# Retry up to this many times when encountering 429s or other server-side errors
MAXIMUM_RETRIES = 2

# Create an output log file?
OUTPUT_LOG = True

# Log file name appended with date and timestamp
LOG_FILE_PREFIX = 'meraki_api_'

# If output log used, output to console too?
PRINT_TO_CONSOLE = True

# Simulate POST/PUT/DELETE calls to prevent changes?
SIMULATE_API_CALLS = False



# API key error
class APIKeyError(Exception):
    def __init__(self):
        self.message = 'Meraki API key needs to be defined'
        super(APIKeyError, self).__init__(self.message)

    def __repr__(self):
        return self.message


# To catch exceptions while making API calls
class APIError(Exception):
    def __init__(self, metadata, response):
        self.response = response
        self.tag = metadata['tags'][0]
        self.operation = metadata['operation']
        self.status = self.response.status_code
        self.reason = self.response.reason
        try:
            self.message = self.response.json()
        except ValueError:
            self.message = self.response.text[:100]
        super(APIError, self).__init__(f'{self.tag}, {self.operation} - {self.status} {self.reason}, {self.message}')

    def __repr__(self):
        return f'{self.tag}, {self.operation} - {self.status} {self.reason}, {self.message}'



class MXStaticRoutes(object):
    def __init__(self, session):
        super(MXStaticRoutes, self).__init__()
        self._session = session
    
    def getNetworkStaticRoutes(self, networkId: str):
        """
        **List the static routes for an MX or teleworker network**
        https://api.meraki.com/api_docs#list-the-static-routes-for-an-mx-or-teleworker-network
        
        - networkId (string)
        """

        metadata = {
            'tags': ['MX static routes'],
            'operation': 'getNetworkStaticRoutes',
        }
        resource = f'/networks/{networkId}/staticRoutes'

        return self._session.get(metadata, resource)

    def createNetworkStaticRoute(self, networkId: str, **kwargs):
        """
        **Add a static route for an MX or teleworker network**
        https://api.meraki.com/api_docs#add-a-static-route-for-an-mx-or-teleworker-network
        
        - networkId (string)
        - name (string): The name of the new static route
        - subnet (string): The subnet of the static route
        - gatewayIp (string): The gateway IP (next hop) of the static route
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['MX static routes'],
            'operation': 'createNetworkStaticRoute',
        }
        resource = f'/networks/{networkId}/staticRoutes'

        body_params = ['name', 'subnet', 'gatewayIp']
        payload = {k: v for (k, v) in kwargs.items() if k in body_params}

        return self._session.post(metadata, resource, payload)

    def getNetworkStaticRoute(self, networkId: str, srId: str):
        """
        **Return a static route for an MX or teleworker network**
        https://api.meraki.com/api_docs#return-a-static-route-for-an-mx-or-teleworker-network
        
        - networkId (string)
        - srId (string)
        """

        metadata = {
            'tags': ['MX static routes'],
            'operation': 'getNetworkStaticRoute',
        }
        resource = f'/networks/{networkId}/staticRoutes/{srId}'

        return self._session.get(metadata, resource)

    def updateNetworkStaticRoute(self, networkId: str, srId: str, **kwargs):
        """
        **Update a static route for an MX or teleworker network**
        https://api.meraki.com/api_docs#update-a-static-route-for-an-mx-or-teleworker-network
        
        - networkId (string)
        - srId (string)
        - name (string): The name of the static route
        - subnet (string): The subnet of the static route
        - gatewayIp (string): The gateway IP (next hop) of the static route
        - enabled (string): The enabled state of the static route
        - fixedIpAssignments (string): The DHCP fixed IP assignments on the static route
        - reservedIpRanges (string): The DHCP reserved IP ranges on the static route
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['MX static routes'],
            'operation': 'updateNetworkStaticRoute',
        }
        resource = f'/networks/{networkId}/staticRoutes/{srId}'

        body_params = ['name', 'subnet', 'gatewayIp', 'enabled', 'fixedIpAssignments', 'reservedIpRanges']
        payload = {k: v for (k, v) in kwargs.items() if k in body_params}

        return self._session.put(metadata, resource, payload)

    def deleteNetworkStaticRoute(self, networkId: str, srId: str):
        """
        **Delete a static route from an MX or teleworker network**
        https://api.meraki.com/api_docs#delete-a-static-route-from-an-mx-or-teleworker-network
        
        - networkId (string)
        - srId (string)
        """

        metadata = {
            'tags': ['MX static routes'],
            'operation': 'deleteNetworkStaticRoute',
        }
        resource = f'/networks/{networkId}/staticRoutes/{srId}'

        return self._session.delete(metadata, resource)

